/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: BrOrgInfoRsp.H 548 2006-03-07 22:50:07Z cesar $ */
/** @file BrOrgInfoRsp.H
 *  @brief EPP BrOrgInfoRsp Class
 */

#ifndef __BR_ORG_INFO_RSP_H__
#define __BR_ORG_INFO_RSP_H__

#include <string>
#include <map>
#include <memory>

#include "libepp_nicbr.H"

#include "ContactInfoRsp.H"

using std::string;
using std::map;
using std::auto_ptr;

LIBEPP_NICBR_NS_BEGIN

/// EPP BrOrgInfoRsp Class
class BrOrgInfoRsp : public ContactInfoRsp
{
public:
  /// Default constructor
  BrOrgInfoRsp(bool reset = true) : ContactInfoRsp(false)
  {  
    if (reset) {
      this->reset();
    }
  } 

  /// Parent copy constructor
  BrOrgInfoRsp(ContactInfoRsp *parent, bool reset = true) :
    ContactInfoRsp(false)
  {
    if (reset) {
      this->reset();
    }
    _roid = parent->get_roid();
    _status_set = parent->get_status_set();
    _clID = parent->get_clID();
    _crID = parent->get_crID();
    _crDate = parent->get_crDate();
    _upID = parent->get_upID();
    _upDate = parent->get_upDate();
    _trDate = parent->get_trDate();
    _common = parent->get_common_data();
    _authInfo = parent->get_authInfo();
    
  }
  
  /// Sets the Organization
  /**
     @param organization Organization Identifier
  */
  void set_organization(const string &organization)
  {
    _organization = organization;
  }

  /// Returns the Organization
  /**
     @return Organization Identifier
  */
  string get_organization() 
  {
    return _organization;
  }
  
  /// Inserts a Contact into the list
  /**
     @param id Contact Id
     @param type Contact Type
  */
  void insert_contact(const string &type, const string &id)
  {
    _contact_list[type] = id;
  }

  /// Returns the Contact List
  /**
     @return Contact List
  */
  map< string, string, less<string> > get_contact_list() 
  {
    return _contact_list;
  }

  /// Sets the Legal Representative
  /**
     @param attorney Local Legal Representative
  */
  void set_attorney(const string &attorney) 
  {
    _attorney = attorney;
  }
  
  /// Returns the Legal Representative
  /**
     @return Local Legal Representative
  */
  string get_attorney() 
  {
    return _attorney;
  }

  /// reset attributes
  void reset() { 
    ContactInfoRsp::reset();
    _organization = "";
    _attorney = "";
    _contact_list.clear();
  }

protected:
  /// Organization Identifier
  string _organization;
  
  /// Human Contacts
  map<string, string, less<string> > _contact_list;

  /// Local Legal Representative
  string _attorney;
  
};

LIBEPP_NICBR_NS_END
#endif //__BR_ORG_INFO_RSP_H__
