/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: BeautifierHandlers.cpp 582 2006-03-15 19:26:32Z milena $ */
/**
   Greedy algorithm: 
   
                 ===================================================
                 | CURRENT EVENT                                   |            
====================================================================
|  FORMER EVENT  | start       | character   |  end                |
====================================================================
|    start       | print_start | print_start | print_start + "/>"  |
--------------------------------------------------------------------
|    character   | print_char  |     X       | print_char +        |
|                |             |             | "<" + name + "/>"   |
--------------------------------------------------------------------
|    end         |     X       |     X       | "<" + name + "/>"   |
--------------------------------------------------------------------     
*/

#include <cctype>
#include <iostream>

#include <xercesc/sax/AttributeList.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>

#include "config.h"

#include "BeautifierHandlers.H"
#include "XmlException.H"

XERCES_CPP_NAMESPACE_USE

LIBEPP_NICBR_NS_BEGIN

BeautifierHandlers::BeautifierHandlers(UserData *user_data) 
{
  _depth = 0;
  _user_data = user_data;
  _last_event = NONE;
}

BeautifierHandlers::~BeautifierHandlers() {}


void BeautifierHandlers::startElement(const XMLCh* const name, 
				      AttributeList&  attributes)
{
  if ((_last_event == START) || (_last_event == CHARACTERS)) {
    _user_data->output_txt += _to_be_printed + ">";
  }

  if (_depth != 0)
    _to_be_printed = "\n";

  for (int i = 0; i < _depth; i++) {
    _to_be_printed += " ";
  }
  _to_be_printed += "<" + str_transcode(name);

  if (attributes.getLength() != 0) {
    for (unsigned int i = 0; i < attributes.getLength(); i++) {
      _to_be_printed += " " + str_transcode(attributes.getName(i)) +
	"=" + str_transcode(attributes.getValue(i));
    }
  }

  _last_event = START;
  _depth++;  
}

void BeautifierHandlers::characters(const XMLCh* const chars, 
				    const unsigned int    length)
{
  // skip full space strings
  string lstr = str_transcode(chars);
  bool allspaces = true;
  for (unsigned int i = 0; i < lstr.length(); i++) {
    if (isspace(lstr.c_str()[i]) == 0) {
      allspaces = false;
      break;
    }
  }

  if (allspaces == false) {
    if (_last_event == START) {
      _user_data->output_txt += _to_be_printed + ">";
    }
    _to_be_printed = lstr;
    _last_event = CHARACTERS;
  }
}

void BeautifierHandlers::endElement(const XMLCh* const name) 
{
  _depth--;

  string whitespace;
  if (_last_event == END) {
    whitespace = "\n";
    for (int i = 0; i < _depth; i++) {
      whitespace += " ";
    }
  }

  switch (_last_event) {
  case START:
    _user_data->output_txt += _to_be_printed + "/>";
    break;
  case CHARACTERS:
    _user_data->output_txt += _to_be_printed + "</" + str_transcode(name) + ">";
    break;
  case END:
    _user_data->output_txt += whitespace + "</" + str_transcode(name) + ">";
    break;
  default:
    break;
  }

  _last_event = END;
}

void BeautifierHandlers::error(const SAXParseException& e)
{
  throw XmlException(XmlException::INVALID_XML_DOCUMENT, 
		     "The XML document is invalid", 
		     str_transcode(e.getMessage()));
}

void BeautifierHandlers::fatalError(const SAXParseException& e)
{
  throw XmlException(XmlException::INVALID_XML_DOCUMENT, 
		     "The XML document is invalid", 
		     str_transcode(e.getMessage()));
}

void BeautifierHandlers::warning(const SAXParseException& e)
{
  throw XmlException(XmlException::INVALID_XML_DOCUMENT, 
		     "The XML document is invalid", 
		     str_transcode(e.getMessage()));
}

string BeautifierHandlers::str_transcode(const XMLCh *const to_transcode)
{
  char *str = XMLString::transcode(to_transcode);
  string ret_str;
  if (str == 0) {
    ret_str = "";
  } else {
    ret_str = str;
  }
  XMLString::release(&str);
  return ret_str;
}

LIBEPP_NICBR_NS_END
