/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: ContactUpdate.cpp 547 2006-03-07 22:44:18Z cesar $ */

#include <set>

#include "config.h"

#include "ContactUpdate.H"
#include "StrUtil.H"

using std::set;

LIBEPP_NICBR_NS_BEGIN

void ContactUpdate::set_xml_template(const string &xml_template)
{
  Action::set_xml_template_common(xml_template);
  map < string, string, less<string> > to_parse;
  
  to_parse["id"] = get_command()->get_common_data().get_id();

  set<string> status_list_add = get_command()->get_status_list_add();
  set<string> status_list_rem = get_command()->get_status_list_rem();
  set<string>::const_iterator it;
  if(status_list_add.size() > 0) {
    to_parse["add"] = "<contact:add>";
    for(it = status_list_add.begin(); it != status_list_add.end(); it++) {
      to_parse["add"] += "<contact:status s='" + (*it) + "'/>";
    }
    to_parse["add"] += "</contact:add>";
  }
  else
    to_parse["add"] = "";

  if(status_list_rem.size() > 0) {
    to_parse["rem"] = "<contact:rem>";
    for(it = status_list_rem.begin(); it != status_list_rem.end(); it++) {
      to_parse["rem"] += "<contact:status s='" + (*it) + "'/>";
    }
    to_parse["rem"] += "</contact:rem>";
  }
  else
    to_parse["rem"] = "";

  bool chg = get_command()->get_chg();
  
  if(chg) {
    vector<CommonData::PostalInfo> postal_info = 
      get_command()->get_common_data().get_postal_info();
    vector<CommonData::PostalInfo>::const_iterator it;
    string pf_str, org, str2, str3, sp, pc;
    
    for (it = postal_info.begin(); it != postal_info.end(); it++) {
      if(strcmp((*it).org.c_str(), "") == 0)
	org = "";
      else
	org = "<contact:org>" + (*it).org + "</contact:org>";
      if(strcmp((*it).str2.c_str(), "") == 0)
	str2 = "";
      else
	str2 = "<contact:street>" + (*it).str2 + "</contact:street>";
      if(strcmp((*it).str3.c_str(), "") == 0)
	str3 = "";
      else
	str3 = "<contact:street>" + (*it).str3 + "</contact:street>";
      if(strcmp((*it).sp.c_str(), "") == 0)
	sp = "";
      else
	sp = "<contact:sp>" + (*it).sp + "</contact:sp>";
      if(strcmp((*it).pc.c_str(), "") == 0)
	pc = "";
      else
	pc = "<contact:pc>" + (*it).pc + "</contact:pc>";
      
      pf_str += "<contact:postalInfo type='" + (*it).type + "'>" +
	"<contact:name>" + (*it).name + "</contact:name>" + org + 
	"<contact:addr><contact:street>" + (*it).str1 + "</contact:street>" + 
	str2 + str3 + "<contact:city>" + (*it).city + "</contact:city>" +
	sp + pc + "<contact:cc>" + (*it).cc + 
	"</contact:cc></contact:addr></contact:postalInfo>";
    }
    
    CommonData::Phone voice;
    voice = get_command()->get_common_data().get_voice();
    string voice_str("");
    if(strcmp(voice.number.c_str(), "") != 0) {
      if(strcmp(voice.ext.c_str(), "") == 0)
	voice_str = "<contact:voice>" + voice.number + "</contact:voice>";
      else
	voice_str = "<contact:voice x='" + voice.ext + "'>" + 
	  voice.number + "</contact:voice>";
    }
    
    CommonData::Phone fax;
    fax = get_command()->get_common_data().get_fax();
    string fax_str("");
    if(strcmp(fax.number.c_str(), "") != 0) {
      if(strcmp(fax.ext.c_str(), "") == 0)
	fax_str = "<contact:fax>" + fax.number + "</contact:fax>";
      else
	fax_str = "<contact:fax x='" + fax.ext + "'>" + 
	  fax.number + "</contact:fax>";
    }
    
    string email = "<contact:email>" + 
      get_command()->get_common_data().get_email() + "</contact:email>";
    
    AuthInfo authInfo;
    authInfo = get_command()->get_authInfo();
    string auth_info_str;
    auth_info_str = "<contact:authInfo>";
    if (strcmp(authInfo.get_roid().c_str(), "") == 0) {
      auth_info_str += "<contact:pw>" + authInfo.get_pw() + "</contact:pw>";
    } else {
      auth_info_str += "<contact:pw roid='" + authInfo.get_roid() + "'>" + 
	authInfo.get_pw() + "</contact:pw>";
    }
    auth_info_str += "</contact:authInfo>";
       
    CommonData::Disclose disclose = 
    get_command()->get_common_data().get_disclose();
    
    string disclose_str;
    if(disclose.name_int == true)
      disclose_str += "<contact:name type =\"int\"/>";
    if(disclose.name_loc == true)
      disclose_str += "<contact:name type=\"loc\"/>";
    if(disclose.org_int == true)
      disclose_str += "<contact:org type=\"int\"/>";
    if(disclose.org_loc == true)
      disclose_str += "<contact:org type=\"loc\"/>";
    if(disclose.addr_int == true)
      disclose_str += "<contact:addr type=\"int\"/>";
    if(disclose.addr_loc == true)
      disclose_str += "<contact:addr type=\"loc\"/>";
    if(disclose.voice == true)
      disclose_str += "<contact:voice/>";
    if(disclose.fax == true)
      disclose_str += "<contact:fax/>";
    if(disclose.email == true)
      disclose_str += "<contact:email/>";
    
    string disclose_total("");
    if(strcmp(disclose_str.c_str(), "") != 0) {
      disclose_total = "<contact:disclose flag='" +  
	StrUtil::to_string("%d", disclose.flag) + "'>" + 
	disclose_str + "</contact:disclose>";
    }
    
    to_parse["chg"] = "<contact:chg>" + pf_str +  voice_str + fax_str +  
      email + auth_info_str + disclose_total + "</contact:chg>";
  }
  else {
    to_parse["chg"] = "";
  }
  
  _xml = StrUtil::parse(_xml, to_parse, "$(", ")$");
}

LIBEPP_NICBR_NS_END
