/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: DomainUpdate.cpp 536 2006-03-03 15:00:29Z cesar $ */

#include "config.h"

#include "DomainUpdate.H"
#include "StrUtil.H"

LIBEPP_NICBR_NS_BEGIN

void DomainUpdate::set_xml_template(const string &xml_template)
{
  Action::set_xml_template_common(xml_template);
  map < string, string, less<string> > to_parse;

  //name
  to_parse["name"] = get_command()->get_name();

  //aux containers and iterators
  set<struct NameServer> ns_cnt;
  set<struct NameServer>::const_iterator it_ns;
  set<NSIPAddr>::const_iterator it_ip;
  map< string, string, less<string> > contact_cnt;
  map< string, string, less<string> >::const_iterator it_contact;
  set<struct DomainUpdateCmd::Status> status_cnt;
  set<struct DomainUpdateCmd::Status>::const_iterator it_status;

  //stuff to add
  ns_cnt = get_command()->get_nameserver_add();
  contact_cnt = get_command()->get_contact_add();
  status_cnt = get_command()->get_status_add();

  if (ns_cnt.size() > 0 ||
      contact_cnt.size() > 0 ||
      status_cnt.size() > 0) {
    
    to_parse["add"] = "<domain:add>";
    
    //nameservers to add
    if (ns_cnt.size() > 0) {
      to_parse["add"] += "<domain:ns>";
      for (it_ns = ns_cnt.begin(); it_ns != ns_cnt.end(); 
	   it_ns++) {
	to_parse["add"] += "<domain:hostAttr>";
	to_parse["add"] += "<domain:hostName>" + it_ns->name + 
	  "</domain:hostName>";
	for (it_ip = it_ns->ips.begin(); it_ip != it_ns->ips.end(); 
	     it_ip++) {
	  to_parse["add"] += "<domain:hostAddr";
	  if ((*it_ip).version != "") {
	    to_parse["add"] += " ip='" + (*it_ip).version + "'";
	  }
	  to_parse["add"] += ">" + (*it_ip).addr + "</domain:hostAddr>";
	}
	to_parse["add"] += "</domain:hostAttr>";
      }
      to_parse["add"] += "</domain:ns>";
    }
    
    //contacts to add
    if (contact_cnt.size() > 0) {
      for (it_contact = contact_cnt.begin(); 
	   it_contact != contact_cnt.end(); it_contact++) {
	to_parse["add"] += "<domain:contact type='" + 
	  (*it_contact).first + "'>" + (*it_contact).second + 
	  "</domain:contact>";
      }
    }
    
    //status to add
    if (status_cnt.size() > 0) {
      for (it_status = status_cnt.begin(); it_status != status_cnt.end();
	   it_status++) {
	to_parse["add"] += "<domain:status s='" + (*it_status).s +
	  "' lang='" + (*it_status).lang + "'>" + (*it_status).msg +
	  "</domain:status>";
      }
    }
    
    to_parse["add"] += "</domain:add>";
  }
  
  //stuff to remove
  ns_cnt = get_command()->get_nameserver_rem();
  contact_cnt = get_command()->get_contact_rem();
  status_cnt = get_command()->get_status_rem();
  
  if (ns_cnt.size() > 0 ||
      contact_cnt.size() > 0 ||
      status_cnt.size() > 0) {
    
    to_parse["rem"] = "<domain:rem>";
    
    //nameservers to remove
    if (ns_cnt.size() > 0) {
      to_parse["rem"] += "<domain:ns>";
      for (it_ns = ns_cnt.begin(); it_ns != ns_cnt.end(); 
	   it_ns++) {
	to_parse["rem"] += "<domain:hostAttr>";
	to_parse["rem"] += "<domain:hostName>" + it_ns->name + 
	  "</domain:hostName>";
	for (it_ip = it_ns->ips.begin(); it_ip != it_ns->ips.end(); 
	     it_ip++) {
	  to_parse["rem"] += "<domain:hostAddr";
	  if ((*it_ip).version != "") {
	    to_parse["rem"] += " ip='" + (*it_ip).version + "'";
	  }
	  to_parse["rem"] += ">" + (*it_ip).addr + "</domain:hostAddr>";
	}
	to_parse["rem"] += "</domain:hostAttr>";
      }
      to_parse["rem"] += "</domain:ns>";
    }

    //contacts to remove
    if (contact_cnt.size() > 0) {
      for (it_contact = contact_cnt.begin();
	   it_contact != contact_cnt.end(); it_contact++) {
	to_parse["rem"] += "<domain:contact type='" + 
	  (*it_contact).first + "'>" + (*it_contact).second + 
	  "</domain:contact>";
      }
    }
    
    //status to remove
    if (status_cnt.size() > 0) {
      for (it_status = status_cnt.begin(); it_status != status_cnt.end();
	   it_status++) {
	to_parse["rem"] += "<domain:status s='" + (*it_status).s + "'/>";
      }
    }

    to_parse["rem"] += "</domain:rem>";
  }
  
  //registrant of tag $(chg)$ and
  string registrant_str = get_command()->get_registrant();
  if (registrant_str != "")
    registrant_str = "<domain:registrant>" + registrant_str
      + "</domain:registrant>";
  
  //auth info of tag $(chg)$
  AuthInfo authInfo;
  authInfo = get_command()->get_authInfo();
  string auth_info_str;
  auth_info_str = "<domain:authInfo>";
  
  if (authInfo.get_roid() == "") {
    auth_info_str += "<domain:pw>" + authInfo.get_pw() + "</domain:pw>";
  } else {
    auth_info_str += "<domain:pw roid='" + authInfo.get_roid() + "'>" + 
      authInfo.get_pw() + "</domain:pw>";
  }
  auth_info_str += "</domain:authInfo>";
  to_parse["chg"] = "<domain:chg>" + registrant_str + auth_info_str + 
    "</domain:chg>";
    
  _xml = StrUtil::parse(_xml, to_parse, "$(", ")$");
}

LIBEPP_NICBR_NS_END
