/* 
 * Copyright (C) 2006 Registro.br. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/* $Id: BrOrgCheckTest.cpp 536 2006-03-03 15:00:29Z cesar $ */
#include <memory>

#include "libepp_nicbr.H"

#include "BrOrgCheckTest.H"
#include "BrOrgCheck.H"
#include "FileUtil.H"
#include "IoException.H"
#include "XmlException.H"

LIBEPP_NICBR_NS_USE

CPPUNIT_TEST_SUITE_REGISTRATION(BrOrgCheckTest);

BrOrgCheckTest::BrOrgCheckTest() {}

BrOrgCheckTest::~BrOrgCheckTest() {}

void BrOrgCheckTest::setUp() {}

void BrOrgCheckTest::tearDown() {}

void BrOrgCheckTest::set_xml_template_test()
{
  string to_be_parsed = 
    "<?xml version='1.0' encoding='UTF-8' standalone='no'?>"
    "<epp xmlns='urn:ietf:params:xml:ns:epp-1.0' "
    "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:epp-1.0 "
    "epp-1.0.xsd'>"
    "<command>"
    "<check>"
    "<contact:check "
    "xmlns:contact='urn:ietf:params:xml:ns:contact-1.0' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:contact-1.0 "
    "contact-1.0.xsd'>"
    "$(id_list)$"
    "</contact:check>"
    "</check>"
    "<extension>"
    "<brorg:check"
    "xmlns:brorg='urn:ietf:params:xml:ns:brorg-1.0' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:brorg-1.0 "
    "brorg-1.0.xsd'>"
    "$(organization_list)$"
    "</brorg:check>"
    "</extension>"
    "$(clTRID)$"
    "</command>"
    "</epp>";


  BrOrgCheck brorg_check;
  BrOrgCheckCmd *brorg_check_cmd = brorg_check.get_command(); 

  brorg_check_cmd->insert_id("e123456");
  brorg_check_cmd->insert_id("e654321");
  brorg_check_cmd->insert_organization("e123456", "043.828.151/0001-45");
  brorg_check_cmd->insert_organization("e654321", "005.506.560/0001-36");

  brorg_check.get_command()->set_clTRID("ABC-12345");
  brorg_check.set_xml_template(to_be_parsed);

  string expected = 
    "<?xml version='1.0' encoding='UTF-8' standalone='no'?>"
    "<epp xmlns='urn:ietf:params:xml:ns:epp-1.0' "
    "xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:epp-1.0 "
    "epp-1.0.xsd'>"
    "<command>"
    "<check>"
    "<contact:check "
    "xmlns:contact='urn:ietf:params:xml:ns:contact-1.0' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:contact-1.0 "
    "contact-1.0.xsd'>"
    "<contact:id>e123456</contact:id>"
    "<contact:id>e654321</contact:id>"
    "</contact:check>"
    "</check>"
    "<extension>"
    "<brorg:check"
    "xmlns:brorg='urn:ietf:params:xml:ns:brorg-1.0' "
    "xsi:schemaLocation='urn:ietf:params:xml:ns:brorg-1.0 "
    "brorg-1.0.xsd'>"
    "<brorg:cd>"
    "<brorg:id>e123456</brorg:id>"
    "<brorg:organization>"
    "043.828.151/0001-45"
    "</brorg:organization>"
    "</brorg:cd>"
    "<brorg:cd>"
    "<brorg:id>e654321</brorg:id>"
    "<brorg:organization>"
    "005.506.560/0001-36"
    "</brorg:organization>"
    "</brorg:cd>"
    "</brorg:check>"
    "</extension>"
    "<clTRID>ABC-12345</clTRID>"
    "</command>"
    "</epp>";

  CPPUNIT_ASSERT_EQUAL(expected, brorg_check.get_xml());

  // tests reset method
  CPPUNIT_ASSERT(!brorg_check_cmd->get_id_list().empty());
  CPPUNIT_ASSERT(!brorg_check_cmd->get_organization_list().empty());
  
  brorg_check_cmd->reset();
  CPPUNIT_ASSERT(brorg_check_cmd->get_id_list().empty());
  CPPUNIT_ASSERT(brorg_check_cmd->get_organization_list().empty());
}

void BrOrgCheckTest::command_test()
{
  BrOrgCheck brorg_check;
  BrOrgCheckCmd *brorg_check_cmd = brorg_check.get_command();

  brorg_check_cmd->insert_id("btw214");
  brorg_check_cmd->insert_id("mrf348");
  brorg_check_cmd->insert_id("sht145");
  
  brorg_check_cmd->insert_organization("btw214", "043.828.151/0001-45");
  brorg_check_cmd->insert_organization("mrf348", "005.506.560/0001-36");
  brorg_check_cmd->insert_organization("sht145", "123.456.789/4321-00");
  
  bool exception_caught = false;
  try {
    string xml_template = 
      FileUtil::read_file("../docs/templates/br_org_check.xml");

    brorg_check.get_command()->set_clTRID("ABC-12345");
    brorg_check.set_xml_template(xml_template);

    DomParser parser;
    parser.enable_validation("../docs/schemas");
    parser.parse_command(brorg_check.get_xml());
  } catch (const IoException &e) {
    exception_caught = true;
    printf("\nIO Exception: code [%d] message [%s]",
	e.get_code(), e.get_msg().c_str());
  } catch (const XmlException &e) {
    exception_caught = true;
    printf("\nXml Exception: code [%d] message [%s] low level message [%s]\n",
	   e.get_code(), e.get_msg().c_str(), e.get_low_level_msg().c_str());
  }

  CPPUNIT_ASSERT(!exception_caught);
}
